from abc import ABCMeta
from collections.abc import Mapping

dic_methods = ["get", "keys", "items", "values"]


class ColMeta(ABCMeta):
    def __getattribute__(cls, attid):
        if attid.startswith("_") or (attid in dic_methods):
            return super().__getattribute__(attid)
        return attid


class ColAssigner(Mapping, metaclass=ColMeta):
    """define functions that create columns in a dataframe
    
    later the class atributes can be used to access the column"""

    def __init__(self):
        self._callables = {}
        self._add_callables()

    def __getitem__(self, key):
        return self._callables[key]

    def __iter__(self):
        for k in self._callables.keys():
            yield k

    def __len__(self):
        return len(self._callables)

    def _add_callables(self):
        for mid in self.__dir__():
            if mid.startswith("_") or (mid in dic_methods):
                continue
            m = getattr(self, mid)
            if callable(m):
                self._callables[mid] = m



import os
import toml
import importlib
from setuptools import find_packages, setup

pytom = toml.load("pyproject.toml")
package_name = pytom["project"]["name"]
author_name = " - ".join(pytom["project"]["authors"])

__version__ = ...
with open(os.path.join(package_name, "_version.py")) as fp:
    exec(fp.read())  # get __version__

with open("README.md") as fp:
    long_description = fp.read()

with open("requirements.txt") as fp:
    requirements = fp.read().strip().split()

if __name__ == "__main__":
    setup(
        name=package_name,
        version=__version__,
        description=pytom["project"]["description"],
        long_description=long_description,
        long_description_content_type="text/markdown",
        license="MIT",
        classifiers=["License :: OSI Approved :: MIT License"],
        url=pytom["project"]["url"],
        keywords=pytom["project"].get("keywords", []),
        author=author_name,
        packages=find_packages(),
        include_package_data=True,
        python_requires=pytom["project"]["python"],
        platforms="any",
        install_requires=requirements,
    )
